#ifndef F_CPU
#define F_CPU 7372800UL
#endif

#include <stdlib.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <util/delay.h>

#include "uart.h"
#include "globals.h"

#define MULTI0 1  // 7 per 250ms

#define UART_BAUD_RATE 38400

uint8_t TIM0_MULTIPLIER = MULTI0;
unsigned int chargePD;

char c_string[16]="";
unsigned int c;
unsigned int ca=1;
uint8_t  num=0;
uint16_t adc_PD;
uint16_t adc_PD_avg;
uint8_t next=0;

void adc_init(void){
// Activate ADC with Prescaler 128
 ADCSRA = _BV(ADEN) | _BV(ADPS0)  | _BV(ADPS1) | _BV(ADPS2);
}

uint16_t start_conversion(int ch){ 

	#define AV 30
	uint8_t i=0;
	uint16_t avgs=0;
	
	ADMUX = ch;	

	for(i=0;i<AV;i++){				
		ADCSRA |= _BV(ADSC);					
		while (ADCSRA & _BV(ADSC)){}
		avgs+=ADCW;
	}
		
	return avgs/AV;
}


ISR(TIMER0_OVF_vect){ 	// Lo uso per aggiornare le righe del display
	TIM0_MULTIPLIER--;
	if(TIM0_MULTIPLIER == 0){
		if(bit_is_clear(PIND,4)){
			c = uart_getc();
			if (c & UART_NO_DATA){}
			else {
				switch (c){
					case 1:
						CLEAR_BIT(PORTD,6);	// RESETTO QUANDO LEGGO LO
						SET_BIT(PORTD,5);	// STATO DELLA BATTERIA
					
						chargePD = (start_conversion(2) - 520)/1.03; // Valore in ADC
						if(chargePD>99 && chargePD<200)
							chargePD=99;		// Se va poco sopra scrive 99%
						else if(chargePD>200)
							chargePD=0;		// Se va poco sotto scrive 0%
						if (next<2 && bit_is_clear(PIND,4)){
							uart_putc(1);
							uart_putc(chargePD);
							next++;
						}						
					break;
					case 2:   // INTEGRO
						CLEAR_BIT(PORTD,5);
						SET_BIT(PORTD,6);
						next=0;
					break;
					case 3:   // HOLD - READ
						CLEAR_BIT(PORTD,6);
						adc_PD = start_conversion(1)/4;
						if(next==0 && bit_is_clear(PIND,4)){
							uart_putc(3);
							uart_putc(adc_PD);
							next=1;
						}if(next==1){
							SET_BIT(PORTD,5);
						}
					break;
/*
					case 4:   // RESETTO
						SET_BIT(PORTD,5);
						CLEAR_BIT(PORTD,6);
						next=0;
					break;
*/
					case 5:
						next=0;
					break;
				}
			}
		}

		TIM0_MULTIPLIER = MULTI0;
	}
}


int main(void)
{

	//configurazione timer/counter 0	
	TIMSK0|=(1<<TOIE0);//overflow vector
  	TCCR0B|=(1<<CS01)|(0<<CS00);// 8 prescaler CS01 1 / NO prescaler CS00 1
 	TCNT0 = 0;


/********************************** CONTROLLO TRASMISSIONE ***************************/
	SET_BIT_AS_OUT(DDRD,7);	// Imposto se metterlo in stand-by o in funzione
	SET_BIT(PORTD,7);	// 0 STAND-BY | 1 WORK
	SET_BIT_AS_OUT(DDRD,3);	// Imposto se pronto a scrivere o ricevere il dato
	CLEAR_BIT(PORTD,3);	// 0 READY | 1 NOT READY
	SET_BIT_AS_IN(DDRD,4);	// Controllo se libero o occupato
//	CLEAR_BIT(PORTD,4);	// 0 FREE | 1 BUSY
/************************************************************************************/
	SET_BIT_AS_OUT(DDRD,2);
	SET_BIT(PORTD,2);
	SET_BIT_AS_IN(DDRB,1);  // CONTROLLO SE LA RICEZIONE E' ANDATA A BUON FINE

	SET_BIT_AS_OUT(DDRB,4); // CONTROLLO DI UN PIN

	SET_BIT_AS_OUT(DDRD,6);	// PIN DI INTEGRAZIONE
	CLEAR_BIT(PORTD,6);
	SET_BIT_AS_OUT(DDRD,5);	// PIN DI RESET
	SET_BIT(PORTD,5);

    uart_init( UART_BAUD_SELECT(UART_BAUD_RATE,F_CPU) ); 
    adc_init();
    sei();
							
	while(1){}

return 0;   
}
